@@ -8,14 +8,12 @@
 #include "currency_p.h"
 #include "kunitconversion_debug.h"
 #include "unit_p.h"
+#include "unitcategory.h"
 #include <KLocalizedString>
-#include <QAtomicInteger>
 #include <QDateTime>
 #include <QDir>
-#include <QEventLoop>
 #include <QFileInfo>
 #include <QLocale>
-#include <QMutex>
 #include <QNetworkAccessManager>
 #include <QNetworkInterface>
 #include <QNetworkReply>
@@ -23,7 +21,6 @@
 #include <QSaveFile>
 #include <QStandardPaths>
 #include <QXmlStreamReader>
-#include <mutex>
 
 using namespace std::chrono_literals;
 
@@ -50,7 +47,10 @@ public:
 
     Value convert(const Value &value, const Unit &toUnit) override;
     bool hasOnlineConversionTable() const override;
-    void syncConversionTable(std::chrono::seconds updateSkipSeconds) override;
+    UpdateJob* syncConversionTable(std::chrono::seconds updateSkipSeconds) override;
+
+    QNetworkReply *m_currentReply = nullptr;
+    bool readConversionTable(const QString &cacheLocation);
     bool m_initialized{false}; //!< indicates if units are prepared from currency table
 };
 
@@ -673,96 +673,90 @@ QDateTime Currency::lastConversionTableUpdate()
     }
 }
 
-void CurrencyCategoryPrivate::syncConversionTable(std::chrono::seconds updateSkipPeriod)
+UpdateJob* CurrencyCategoryPrivate::syncConversionTable(std::chrono::seconds updateSkipPeriod)
 {
-    // sync call is expected to be guarded as being called only once
-    auto updateCurrencyConversionTable = [this](const QString &cachePath, bool performNetworkSync) {
-        qCDebug(LOG_KUNITCONVERSION) << "currency conversion table sync started";
-        static QMutex mutex;
-        QMutexLocker locker(&mutex);
-        bool updateError{false};
-        if (performNetworkSync) {
-            if (isConnected()) {
-                // Bug 345750: QNetworkReply does not work without an event loop and doesn't implement waitForReadyRead()
-                QEventLoop loop;
-                QNetworkAccessManager manager;
-                auto *reply = manager.get(QNetworkRequest(QUrl(QString::fromLatin1(URL)))); // reply is owned by the network access manager
-                QObject::connect(reply, &QNetworkReply::finished, [&] {
-                    if (!reply->error()) {
-                        QFileInfo info(cachePath);
-                        const QString cacheDir = info.absolutePath();
-                        if (!QFileInfo::exists(cacheDir)) {
-                            QDir().mkpath(cacheDir);
-                        }
-
-                        QSaveFile cacheFile(cachePath);
-                        if (cacheFile.open(QFile::WriteOnly)) {
-                            cacheFile.write(reply->readAll());
-                            if (!cacheFile.commit()) {
-                                updateError = true;
-                            } else {
-                                qCInfo(LOG_KUNITCONVERSION) << "currency conversion table data obtained via network";
-                            }
-                        }
-                    } else {
-                        updateError = true;
-                        qCCritical(LOG_KUNITCONVERSION) << "currency conversion table network error" << reply->error();
-                    }
-                    loop.quit();
-                });
-                loop.exec(QEventLoop::ExcludeUserInputEvents);
-            } else {
-                qCInfo(LOG_KUNITCONVERSION) << "currency conversion table update has no network connection, abort update";
-            }
-        }
-
-        if (!updateError) {
-            QFile file(cachePath);
-            if (file.open(QIODevice::ReadOnly)) {
-                QXmlStreamReader xml(&file);
-                while (!xml.atEnd()) {
-                    xml.readNext();
-
-                    if (xml.isStartElement() && xml.name() == QLatin1String("Cube")) {
-                        const auto attributes = xml.attributes();
-                        if (attributes.hasAttribute(QLatin1String("currency"))) {
-                            Unit unit = m_unitMap.value(attributes.value(QLatin1String("currency")).toString());
-                            if (unit.isValid()) {
-                                const auto multiplier = attributes.value(QLatin1String("rate")).toDouble();
-                                if (!qFuzzyIsNull(multiplier)) {
-                                    unit.setUnitMultiplier(1.0 / multiplier);
-                                    qCDebug(LOG_KUNITCONVERSION()) << "currency updated:" << unit.description() << multiplier;
-                                }
-                            }
-                        }
-                    }
-                }
-                updateError = xml.hasError();
-                if (updateError) {
-                    qCCritical(LOG_KUNITCONVERSION) << "currency conversion fetch could not parse obtained XML, update aborted";
-                }
-            }
-        }
-        m_initialized = !updateError;
-        return !updateError;
-    };
-
     QFileInfo info(cacheLocation());
-    if (!info.exists() || info.lastModified().secsTo(QDateTime::currentDateTime()) > updateSkipPeriod.count()) {
-        updateCurrencyConversionTable(cacheLocation(), true);
-    } else if (!m_initialized) {
-        updateCurrencyConversionTable(cacheLocation(), false);
+    if (info.exists() && info.lastModified().secsTo(QDateTime::currentDateTime()) <= updateSkipPeriod.count()) {
+        return nullptr; // already present and up to date
     }
+    if (!isConnected()) {
+        qCInfo(LOG_KUNITCONVERSION) << "currency conversion table update has no network connection, abort update";
+        return nullptr;
+    }
+    if (m_currentReply) {
+        return makeUpdateJob(m_currentReply);
+    }
+
+    qCDebug(LOG_KUNITCONVERSION) << "currency conversion table sync started";
+    m_currentReply = nam()->get(QNetworkRequest(QUrl(QString::fromLatin1(URL))));
+    QObject::connect(m_currentReply, &QNetworkReply::finished, [this] {
+        auto reply = m_currentReply;
+        m_currentReply = nullptr;
+        reply->deleteLater();
+        if (reply->error()) {
+            qCWarning(LOG_KUNITCONVERSION) << "currency conversion table network error" << reply->errorString();
+            return;
+        }
+        const auto cachePath = cacheLocation();
+        QFileInfo info(cachePath);
+        const QString cacheDir = info.absolutePath();
+        if (!QFileInfo::exists(cacheDir)) {
+            QDir().mkpath(cacheDir);
+        }
+
+        QSaveFile cacheFile(cachePath);
+        if (!cacheFile.open(QFile::WriteOnly)) {
+            qCCritical(LOG_KUNITCONVERSION) << cacheFile.errorString();
+            return;
+        }
+        cacheFile.write(reply->readAll());
+        if (!cacheFile.commit()) {
+            qCCritical(LOG_KUNITCONVERSION) << cacheFile.errorString();
+            return;
+        }
+        qCInfo(LOG_KUNITCONVERSION) << "currency conversion table data obtained via network";
+        m_initialized = readConversionTable(cachePath);
+    });
+
+    return makeUpdateJob(m_currentReply);
+}
+
+bool CurrencyCategoryPrivate::readConversionTable(const QString &cachePath)
+{
+    QFile file(cachePath);
+    if (!file.open(QIODevice::ReadOnly)) {
+        return false;
+    }
+    QXmlStreamReader xml(&file);
+    while (!xml.atEnd()) {
+        xml.readNext();
+
+        if (xml.isStartElement() && xml.name() == QLatin1String("Cube")) {
+            const auto attributes = xml.attributes();
+            if (attributes.hasAttribute(QLatin1String("currency"))) {
+                Unit unit = m_unitMap.value(attributes.value(QLatin1String("currency")).toString());
+                if (unit.isValid()) {
+                    const auto multiplier = attributes.value(QLatin1String("rate")).toDouble();
+                    if (!qFuzzyIsNull(multiplier)) {
+                        unit.setUnitMultiplier(1.0 / multiplier);
+                        qCDebug(LOG_KUNITCONVERSION()) << "currency updated:" << unit.description() << multiplier;
+                    }
+                }
+            }
+        }
+    }
+
+    if (xml.hasError()) {
+        qCCritical(LOG_KUNITCONVERSION) << "currency conversion fetch could not parse obtained XML, update aborted";
+        return false;
+    }
+    return true;
 }
 
 Value CurrencyCategoryPrivate::convert(const Value &value, const Unit &to)
 {
-    // TODO KF6 remove this blocking call and change behavior that explicit call to syncConversionTable is mandatory before
-    // first access to converted data, also to make syncs more explicit
-    static QMutex updateFlag;
-    {
-        QMutexLocker locker(&updateFlag);
-        CurrencyCategoryPrivate::syncConversionTable(24h);
+    if (!m_initialized) {
+        m_initialized = readConversionTable(cacheLocation());
     }
 
     Value v = UnitCategoryPrivate::convert(value, to);
